@@ -1,4 +1,4 @@
-# Copyright (C) 2001,2002 by the Free Software Foundation, Inc.
+# Copyright (C) 2001-2018 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,7 +12,7 @@
 # 
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software 
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 """MailList mixin class managing the membership pseudo-options.
 """
@@ -30,5 +30,7 @@ class Membership:
             return [('list',   _('Membership&nbsp;List')),
                     ('add',    _('Mass&nbsp;Subscription')),
                     ('remove', _('Mass&nbsp;Removal')),
+                    ('change', _('Address&nbsp;Change')),
+                    ('sync',   _('Sync&nbsp;Membership&nbsp;List')),
                     ]
         return None
